@@ -17,37 +17,44 @@
    along with GNU Sqltutor.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* 
- * $Id: get_new_question.cpp,v 1.3 2009/04/01 18:12:37 cepek Exp $ 
- */
-
 #include "sqltutor.h"
 
 void SQLtutor::get_new_question(pqxx::work& tran) 
 {
   using pqxx::result;
 
-  result res1(tran.exec("SELECT next_tutorial_id, next_question_id"
-                        "  FROM next_question(" 
-                        + session_id + ", '" + hash + "')" ));
-  if (res1.begin()[0].is_null() || res1.begin()[1].is_null())
+  const std::string getnext =
+    "SELECT dataset_id, problem_id, q_ord, language_id "
+    "  FROM next_question(" + session_id + ", '" + hash + "')";
+
+  result res1(tran.exec(getnext));
+  result::const_iterator res = res1.begin();
+
+  if (res[0].is_null())
     {
-      question_id.clear();
+      dataset_id.clear();
+      problem_id.clear();
+      q_ord.clear();
+      language_id.clear();
       throw AllQuestionsDone();
     }
 
-  tutorial_id = res1.begin()[0].as(std::string());
-  question_id = res1.begin()[1].as(std::string());
+  dataset_id  = res["dataset_id" ].as(std::string());
+  problem_id  = res["problem_id" ].as(std::string());
+  q_ord       = res["q_ord"      ].as(std::string());
+  language_id = res["language_id"].as(std::string());
 
   const std::string insert =
-    "INSERT INTO sessions_answers "
-    "   (session_id, tutorial_id, question_id, time) VALUES (" 
-    + session_id + ", " + tutorial_id + ", " + question_id + ", now() )";
-
+    "INSERT INTO sessions_questions "
+    "  (session_id, dataset_id, problem_id, q_ord, language_id, time) VALUES (" 
+    + session_id + ", " + dataset_id + ", " + problem_id + ", " 
+    + q_ord + ", '" + language_id + "', now() )";
+  
   using namespace pqxx;
   connection c( db_connection );
   work       t(c, "get_new_question");
   set_schema(t);
   result     r(t.exec(insert));
   t.commit();
+
 }
