@@ -1,26 +1,22 @@
-/* 
+/*
    This file is part of GNU Sqltutor
-   Copyright (C) 2008  Free Software Foundation, Inc.
+   Copyright (C) 2008, 2013  Free Software Foundation, Inc.
    Contributed by Ales Cepek <cepek@gnu.org>
- 
+
    GNU Sqltutor is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    GNU Sqltutor is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with GNU Sqltutor.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* 
- * $Id: settings.cpp,v 1.5 2009/04/01 18:12:38 cepek Exp $ 
- */
-
 #include "sqltutor.h"
 #include "config.h"
 
@@ -54,6 +50,7 @@ T SQLtutor::main_data         = "Display data";
 T SQLtutor::main_sql          = "Execute SQL";
 T SQLtutor::remaining_rows    = "Remaining rows not displayed ...";
 T SQLtutor::new_test          = "New test";
+T SQLtutor::reload_page       = "Reload the page";
 T SQLtutor::test_finished     = "Test finished ...";
 T SQLtutor::t_tutorial        = "Tutorial";
 T SQLtutor::t_user            = "User";
@@ -66,6 +63,8 @@ T SQLtutor::t_table           = "Table";
 T SQLtutor::t_columns         = "Columns";
 T SQLtutor::t_bad_value_min   = "Bad value min";
 T SQLtutor::t_bad_value_max   = "Bad value max";
+T SQLtutor::t_bad_value_dset  = "Unknown dataset";
+T SQLtutor::t_bad_value_char  = "Special characters are not allowed";
 T SQLtutor::t_correct_answer  = "Correct answer";
 T SQLtutor::t_wrong_answer    = "Wrong answer";
 T SQLtutor::t_missing_answer  = "Missing answer";
@@ -85,10 +84,12 @@ T SQLtutor::t_unknown_state   = "unknown internal state";
 T SQLtutor::t_session         = "session ";
 T SQLtutor::t_session_unknown = "Unknown session ID ... ";
 T SQLtutor::t_session_closed  = "Session has been already closed ... ";
-T SQLtutor::t_select_tutorial = "Please, select tutorial first ... ";
+T SQLtutor::t_select_tutorial = "Please, select a tutorial first ... ";
 T SQLtutor::t_no_datasets     = "No datasets available ... ";
 T SQLtutor::t_empty_set       = "empty set";
 T SQLtutor::t_welcome         = "Welcome to GNU Sqltutor";
+T SQLtutor::t_geometry        = "geometry";
+T SQLtutor::t_multiple_sessions = "Multiple sessions from the same host";
 
 
 /********************************************************************/
