@@ -1,26 +1,22 @@
-/* 
+/*
    This file is part of GNU Sqltutor
-   Copyright (C) 2008  Free Software Foundation, Inc.
+   Copyright (C) 2008, 2013  Free Software Foundation, Inc.
    Contributed by Ales Cepek <cepek@gnu.org>
- 
+
    GNU Sqltutor is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    GNU Sqltutor is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with GNU Sqltutor.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* 
- * $Id: sqltutor.h,v 1.8 2009/04/01 18:12:38 cepek Exp $ 
- */
-
 #ifndef __h___SQLTUTOR_H___sqltutor_h___SQLtutor
 #define __h___SQLTUTOR_H___sqltutor_h___SQLtutor
 
@@ -40,13 +36,6 @@ public:
   ~SQLtutor();
 
   void run();
-  
-  template<typename Form> static void 
-  display_answers(Form&,  pqxx::work& tran, std::string session_id);
-
-  template<typename Form> static void 
-  display_question(Form&,  pqxx::work& tran, 
-                   std::string tutorial_id, std::string question_id);
 
 private:
 
@@ -65,6 +54,7 @@ private:
   static const std::string main_sql;
   static const std::string remaining_rows;
   static const std::string new_test;
+  static const std::string reload_page;
   static const std::string test_finished;
   static const std::string t_tutorial;
   static const std::string t_user;
@@ -77,6 +67,8 @@ private:
   static const std::string t_columns;
   static const std::string t_bad_value_min;
   static const std::string t_bad_value_max;
+  static const std::string t_bad_value_dset;
+  static const std::string t_bad_value_char;
   static const std::string t_correct_answer;
   static const std::string t_wrong_answer;
   static const std::string t_missing_answer;
@@ -100,6 +92,9 @@ private:
   static const std::string t_no_datasets;
   static const std::string t_empty_set;
   static const std::string t_welcome;
+  static const std::string t_geometry;
+  static const std::string t_multiple_sessions;
+
 
   // program settings
 
@@ -109,21 +104,23 @@ private:
   static const std::string init_state;
   static const std::string main_state;
 
-  class Exception : public std::exception   
+  class Exception : public std::exception
   {
   public:
     const char* what() { return "SQLtutor : Base Exception";     }
   };
-  
-  class AllQuestionsDone : public Exception 
+
+  class AllQuestionsDone : public Exception
   {
     const char* what() { return "SQLtutor : All Questions Done"; }
   };
 
+  enum { max_row_count = 30 };
+
   CGI           cgi;
   Form          form;
-  size_t        sql_result_size; 
-  size_t        sql_result_columns; 
+  size_t        sql_result_size;
+  size_t        sql_result_columns;
   size_t        sql_tutor_size;
   size_t        sql_tutor_columns;
   bool          correct_answer;
@@ -131,10 +128,14 @@ private:
   std::string   sql;
   std::string   sql_checked;
   std::string   tutorial_id;
-  std::string   question_id;
+  std::string   dataset_id;
+  std::string   problem_id;
+  std::string   q_ord;
+  std::string   language_id;
   std::string   answered;
   std::string   session_id;
   std::string   hash;
+  std::string   all_qasked;
 
   void form_init();
   void form_main();
@@ -147,7 +148,7 @@ private:
   void save_answer     (pqxx::work& transaction);
   void set_schema      (pqxx::work& transaction)
   {
-    transaction.exec("SET search_path TO sqltutor;");
+    transaction.exec("SET search_path TO sqltutor, sqltutor_data, public;");
   }
 
   void show_sql_result  ();
@@ -156,7 +157,19 @@ private:
   std::string button_sep() { return "&nbsp;&nbsp;&nbsp;&nbsp;"; }
   bool empty            (std::string);
 
-  std::string tutorial_selection();
+  Select tutorial_selection();
+
+  void display_answers (Form&,  pqxx::work& tran, std::string session_id);
+  void display_question(Form&,  pqxx::work& tran,
+                        std::string dataset_id,
+                        std::string problem_id,
+                        std::string q_ord,
+                        std::string language_id);
+
+  // postgis
+
+  pqxx::oid geom_oid;
+
 };
 
 
